using System;
using System.IO;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using ZedGraph;

namespace FFTChart
{
    public class FFTChart : Form
    {
        #region Declarations
        private MenuStrip m_menu;
        private FFTChartGraphControl m_graph;
        private Timer m_tmrMenuDelay;
        private ToolStripComboBox m_cboType;
        private Timer m_tmrTrackChanges = new Timer();
        private ToolStripMenuItem m_fileTrackMenuItem;

        // Detections strings
        private const string DISTORSIONTAG = "Total Harmonic Distortion:";
        private string[] DATATAGS = new string[] { "Number", "[Hz]", "Component" };

        public enum enumMenuIds
        {
            File,
            FileOpen,
            FileTrackChanges,
            FileQuit,
            View,
            ViewClear,
        }
        #endregion

        #region Constructor
        public FFTChart()
        {
            // Create graph
            m_graph = new FFTChartGraphControl();
            m_graph.BorderStyle = BorderStyle.None;
            m_graph.AllowDrop = true;
            m_graph.DragDrop += new DragEventHandler(m_graph_DragDrop);
            m_graph.DragEnter += new DragEventHandler(m_graph_DragEnter);
            m_graph.DragOver += new DragEventHandler(m_graph_DragOver);

            // Create menu
            m_menu = new MenuStrip();
            m_menu.Dock = DockStyle.Top;

            ToolStripMenuItem fileMenuItem = new ToolStripMenuItem();
            fileMenuItem.Text = "&File";
            fileMenuItem.Tag = enumMenuIds.File;
            fileMenuItem.DropDownItemClicked += new ToolStripItemClickedEventHandler(OnDropDownItemClicked);
            m_menu.Items.Add(fileMenuItem);

            ToolStripMenuItem fileOpenMenuItem = new ToolStripMenuItem();
            fileOpenMenuItem.Text = "&Open";
            fileOpenMenuItem.ShortcutKeys = Keys.Control | Keys.O;
            fileOpenMenuItem.Tag = enumMenuIds.FileOpen;
            fileMenuItem.DropDownItems.Add(fileOpenMenuItem);

            fileMenuItem.DropDownItems.Add(new ToolStripSeparator());

            m_fileTrackMenuItem = new ToolStripMenuItem();
            m_fileTrackMenuItem.Text = "&Track Changes";
            m_fileTrackMenuItem.Tag = enumMenuIds.FileTrackChanges;
            m_fileTrackMenuItem.Checked = false;
            fileMenuItem.DropDownItems.Add(m_fileTrackMenuItem);

            fileMenuItem.DropDownItems.Add(new ToolStripSeparator());

            ToolStripMenuItem fileQuitMenuItem = new ToolStripMenuItem();
            fileQuitMenuItem.Text = "&Quit";
            fileQuitMenuItem.Tag = enumMenuIds.FileQuit;
            fileMenuItem.DropDownItems.Add(fileQuitMenuItem);

            ToolStripMenuItem viewMenuItem = new ToolStripMenuItem();
            viewMenuItem.Text = "&View";
            viewMenuItem.Tag = enumMenuIds.View;
            viewMenuItem.DropDownItemClicked += new ToolStripItemClickedEventHandler(OnDropDownItemClicked);
            m_menu.Items.Add(viewMenuItem);

            ToolStripMenuItem viewClearMenuItem = new ToolStripMenuItem();
            viewClearMenuItem.Text = "&Clear";
            viewClearMenuItem.ShortcutKeys = Keys.Delete;
            viewClearMenuItem.Tag = enumMenuIds.ViewClear;
            viewMenuItem.DropDownItems.Add(viewClearMenuItem);

            m_cboType = new ToolStripComboBox();
            m_cboType.DropDownStyle = ComboBoxStyle.DropDownList;
            m_cboType.Alignment = ToolStripItemAlignment.Right;
            m_cboType.Padding = new Padding(5, 0, 0, 10);
            m_cboType.Items.Add("Fourier Component");
            m_cboType.Items.Add("Normalized Component");
            m_cboType.Items.Add("Phase Degree");
            m_cboType.Items.Add("Normalized PhaseDegree");
            m_cboType.SelectedIndex = 0;
            m_cboType.Size = new Size(200, 20);
            m_cboType.SelectedIndexChanged += new EventHandler(m_cboType_SelectedIndexChanged);
            m_menu.Items.Add(m_cboType);

            m_tmrMenuDelay = new Timer();
            m_tmrMenuDelay.Interval = 20;
            m_tmrMenuDelay.Tick += new EventHandler(m_tmrMenuDelay_Tick);

            m_tmrTrackChanges = new Timer();
            m_tmrTrackChanges.Enabled = false;
            m_tmrTrackChanges.Tick += new EventHandler(m_tmrTrackChanges_Tick);

            this.Text = "FFT Curve Tracer for LTSpice";
            this.Controls.AddRange(new Control[] { m_menu, m_graph });
            this.MainMenuStrip = m_menu;
            this.Size = new Size(630, 420);
            this.AllowDrop = true;

            string[] args = System.Environment.GetCommandLineArgs();
            if (args.Length > 1)
            {
                for (int i = 1; i < args.Length; i++)
                {
                    if (File.Exists(args[i]))
                        ExtractFFTInfos(args[i]);
                }
            }
        }
        #endregion

        #region Events
        void OnDropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            // Wait for timer during the menu close!!!
            m_tmrMenuDelay.Tag = e.ClickedItem.Tag;
            m_tmrMenuDelay.Stop();
            m_tmrMenuDelay.Start();
        }

        void m_tmrMenuDelay_Tick(object sender, EventArgs e)
        {
            m_tmrMenuDelay.Stop();
            switch ((enumMenuIds)m_tmrMenuDelay.Tag)
            {
                case enumMenuIds.FileOpen:
                    ExtractFFTInfos();
                    break;

                case enumMenuIds.FileTrackChanges:
                    m_tmrTrackChanges.Interval = 1;
                    m_tmrTrackChanges.Enabled = !m_tmrTrackChanges.Enabled;
                    m_fileTrackMenuItem.Checked = m_tmrTrackChanges.Enabled;
                    break;

                case enumMenuIds.FileQuit:
                    this.Close();
                    break;

                case enumMenuIds.ViewClear:
                    m_graph.Clear();
                    break;

            }
        }

        protected override void OnShown(EventArgs e)
        {
            base.OnShown(e);
            System.Windows.Forms.Application.DoEvents();
            if (m_graph.GraphPane.CurveList.Count == 0)
                ExtractFFTInfos();
        }

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);

            m_graph.Location = new Point(0, m_menu.Height);
            m_graph.Size = new Size(this.ClientRectangle.Width, this.ClientRectangle.Height - m_menu.Height);
        }

        private void m_graph_DragEnter(object sender, DragEventArgs e)
        {
            OnDragEnter(e);
        }

        protected override void OnDragEnter(DragEventArgs e)
        {
            e.Effect = AllowDragAndDrop(e);
        }

        private void m_graph_DragOver(object sender, DragEventArgs e)
        {
            OnDragOver(e);
        }

        protected override void OnDragOver(DragEventArgs e)
        {
            e.Effect = AllowDragAndDrop(e);
        }

        private void m_graph_DragDrop(object sender, DragEventArgs e)
        {
            OnDragDrop(e);
        }

        protected override void OnDragDrop(DragEventArgs e)
        {
            IDataObject IData = e.Data;
            string[] strFile = null;

            if (IData.GetDataPresent("FileDrop", false))
                strFile = (string[])(IData.GetData("FileDrop", false));
            else if (IData.GetDataPresent("FileNameW", false))
                strFile = (string[])(IData.GetData("FileNameW", false));
            else if (IData.GetDataPresent("FileName", false))
                strFile = (string[])(IData.GetData("FileName", false));

            if (strFile != null && strFile.Length > 0)
            {
                for (int i = 0; i < strFile.Length; i++)
                    ExtractFFTInfos(strFile[i]);
            }
        }

        private void m_cboType_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_graph.GraphType = (FFTChartGraphControl.enumGraphTypes)m_cboType.SelectedIndex;
        }

        private void m_tmrTrackChanges_Tick(object sender, EventArgs e)
        {
            try
            {
                m_tmrTrackChanges.Stop();
                string message = "The following files are changed: \r\n";
                List<HarmonicCollection> dataToRefresh = new List<HarmonicCollection>();

                foreach (HarmonicCollection data in m_graph.DataSet)
                {
                    FileInfo fileInfo = new FileInfo(data.FileName);
                    if (fileInfo.LastWriteTime != data.FileVersion)
                    {
                        // Try to open the file
                        Stream stm = null;
                        try
                        {
                            stm = fileInfo.OpenRead();
                        }
                        catch
                        {

                        }
                        if (stm != null)
                        {
                            message += data.FileName + "\r\n";
                            data.FileVersion = fileInfo.LastWriteTime;
                            dataToRefresh.Add(data);
                            stm.Close();
                        }
                    }
                }

                if (dataToRefresh.Count > 0)
                {
                    message += "\r\nClick yes to reload the existing chart.\r\nClick no to reload a new chart\r\nClick cancel to keep the existing chart";


                    DialogResult result = MessageBox.Show(this, message, this.Text, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);
                    switch (result)
                    {
                        case DialogResult.Yes:
                            foreach (HarmonicCollection data in dataToRefresh)
                            {
                                try
                                {
                                    ExtractFFTInfos(data);
                                }
                                catch (Exception ex)
                                {
                                    MessageBox.Show(this, "Unable to read FFT datas on this file: " + ex.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                                    return;
                                }

                                if (data.Count > 0)
                                    m_graph.ReloadData(data);
                                else
                                    MessageBox.Show(this, "Unable to found FFT datas on this file.", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            }
                            m_graph.AxisChange();
                            break;

                        case DialogResult.No:
                            foreach (HarmonicCollection data in dataToRefresh)
                                ExtractFFTInfos(data.FileName);
                            break;
                    }
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.Assert(false, ex.ToString());
            }
            finally
            {
                m_tmrTrackChanges.Interval = 2000;
                m_tmrTrackChanges.Start();
            }
        }
        #endregion

        #region Functions
        public void ExtractFFTInfos()
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.CheckFileExists = true;
            dlg.Filter = "Log files (*.log)|*.log|All files (*.*)|*.*";
            dlg.RestoreDirectory = true;
            dlg.Multiselect = true;
            dlg.Title = "Open log file containing FFT Informations";

            if (dlg.ShowDialog(this) == DialogResult.OK)
            {
                for (int i=0; i<dlg.FileNames.Length; i++)
                    ExtractFFTInfos(dlg.FileNames[i]);
            }
        }

        public void ExtractFFTInfos(string fileName)
        {
            HarmonicCollection datas = new HarmonicCollection();
            datas.FileName = fileName;
            
            try
            {
                ExtractFFTInfos(datas);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Unable to read FFT datas on this file: " + ex.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            if (datas.Count > 0)
                m_graph.AddData(datas);
            else
                MessageBox.Show(this, "Unable to found FFT datas on this file.", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }
        
        private void ExtractFFTInfos(HarmonicCollection datas)
        {
            // Open log file for read
            StreamReader stmr = null;
            FileInfo fileInfo = new FileInfo(datas.FileName);
            datas.FileVersion = fileInfo.LastWriteTime;
            datas.Clear();

            try
            {
                stmr = new StreamReader(datas.FileName, System.Text.Encoding.Default);
                string line = stmr.ReadLine();
                bool ack = false;

                while (line != null)
                {
                    if (ack)
                    {
                        if (line.Contains(DISTORSIONTAG))
                        {
                            line = line.Substring(DISTORSIONTAG.Length);
                            datas.TotalHarmonicDistorsion = double.Parse(line.Replace("%", "")) / 100;
                            break;
                        }
                        else
                        {
                            // Parse current line
                            Harmonic h = new Harmonic();
                            string[] values = line.Split('\t');
                            int i = 0;
                            foreach (string s in values)
                            {
                                string str = s.Replace("", "");
                                double value;
                                if (double.TryParse(str, out value))
                                {
                                    switch (i)
                                    {
                                        case 0:
                                            h.HarmonicNumber = value;
                                            break;
                                        case 1:
                                            h.Frequency = value;
                                            break;
                                        case 2:
                                            h.FourierComponent = value;
                                            break;
                                        case 3:
                                            h.NormalizedComponent = value;
                                            break;
                                        case 4:
                                            h.PhaseDegree = value;
                                            break;
                                        case 5:
                                            h.NormalizedPhaseDegree = value;
                                            break;
                                    }
                                    i++;
                                }
                            }
                            datas.Add(h);
                        }
                    }
                    else if (line.Contains(DATATAGS[0]))
                    {
                        ack = true;
                        for (int i = 1; i < DATATAGS.Length; i++)
                        {
                            if (line.Contains(DATATAGS[i]) == false)
                            {
                                ack = false;
                                break;
                            }
                        }
                    }
                    line = stmr.ReadLine();
                }
            }
            finally
            {
                if (stmr != null)
                    stmr.Close();
            }
        }

        private DragDropEffects AllowDragAndDrop(DragEventArgs e)
        {
            IDataObject IData = e.Data;
            string[] strFile;
            FileInfo fil;

            if (IData.GetDataPresent("FileName", false))
            {
                strFile = (string[])(IData.GetData("FileName", false));
                if (strFile.Length == 1)
                {
                    fil = new FileInfo(strFile[0]);
                    if (fil.Exists)
                        return DragDropEffects.Copy;
                }
            }
            return DragDropEffects.None;

        }
        #endregion
    }

    public class HarmonicCollection : List<Harmonic>
    {
        public double TotalHarmonicDistorsion = 0;
        public string FileName = "";
        public DateTime FileVersion = DateTime.Now;
    }

    public class Harmonic
    {
        public double HarmonicNumber = 0;
        public double Frequency = 0;
        public double FourierComponent = 0;
        public double NormalizedComponent = 0;
        public double PhaseDegree = 0;
        public double NormalizedPhaseDegree = 0;
    }
}