using System;
using System.Drawing;
using System.Collections.Generic;
using System.Reflection;
using System.Windows.Forms;
using ZedGraph;

namespace FFTChart
{
    #region GraphClass
    public class FFTChartGraphControl : ZedGraphControl
    {
        #region Declarations
        private Color[] m_graphColors = new Color[] { Color.Orange, Color.Blue, Color.Red, Color.Green, Color.Gray, Color.LightSalmon };
        private int m_currentColor = 0;
        private List<HarmonicCollection> m_dataSet = new List<HarmonicCollection>();
        private double m_minYAxis = 0;
        private double m_maxYAxis = 0;
        private enumGraphTypes m_graphType = enumGraphTypes.FourierComponent;
        private ToolTip m_toolTip;

        public enum enumGraphTypes
        {
            FourierComponent,
            NormalizedComponent,
            PhaseDegree,
            NormalizedPhaseDegree,
        }
        #endregion

        #region Constructor
        public FFTChartGraphControl()
        {
            GraphPane pane = this.GraphPane;
            pane.Title.Text = "";

            XAxis x = pane.XAxis;
            x.Title.Text = "Frequency";
            x.Scale.MinAuto = false;
            x.Scale.MaxAuto = false;
            x.Scale.FormatAuto = false;

            YAxis y = pane.YAxis;
            y.Scale.MinAuto = false;
            y.Scale.MaxAuto = false;
            y.Scale.FormatAuto = false;
            y.Scale.IsUseTenPower = false;

            // Fill the axis background with a color gradient
            pane.Chart.Fill = new Fill(Color.White, Color.LightGoldenrodYellow, 45F);

            // Fill the pane background with a color gradient
            pane.Fill = new Fill(ProfessionalColors.MenuStripGradientEnd, ProfessionalColors.MenuStripGradientBegin, 180F);

            // Calculate the Axis Scale Ranges
            this.AxisChange();

            m_toolTip = new ToolTip();
            m_toolTip.InitialDelay = 1000;
        }
        #endregion

        #region Properties
        public enumGraphTypes GraphType
        {
            get
            {
                return m_graphType;
            }
            set
            {
                if (m_graphType != value)
                {
                    m_graphType = value;
                    // Reload datas
                    this.DataSet = m_dataSet;
                }
            }
        }

        public List<HarmonicCollection> DataSet
        {
            get
            {
                return m_dataSet;
            }
            set
            {
                this.GraphPane.CurveList.Clear();
                m_currentColor = 0;
                foreach (HarmonicCollection data in value)
                {
                    FillData(data);
                }
                m_dataSet = value;
                this.AxisChange();
            }
        }
        #endregion

        #region Overrides
        protected override void OnMouseMove(MouseEventArgs e)
        {
            base.OnMouseMove(e);

            CurveItem curve;
            int pointIndex;
            if (this.GraphPane.FindNearestPoint(e.Location, out curve, out pointIndex) && curve[pointIndex].Tag != null)
            {
                if (m_toolTip.Tag != curve[pointIndex])
                {
                    m_toolTip.Hide(this);
                    m_toolTip.Tag = curve[pointIndex];
                    m_toolTip.Show(curve[pointIndex].Tag.ToString(), this, e.X, e.Y - 40);
                }
            }
            else
            {
                m_toolTip.Tag = null;
                m_toolTip.Hide(this);
            }
        }

        protected override void OnMouseWheel(MouseEventArgs e)
        {
            double value = this.GraphPane.YAxis.Scale.Max;
            double unit = (m_maxYAxis - m_minYAxis) / 100;
            if (value < m_maxYAxis || e.Delta > 0)
            {
                value -= e.Delta * unit / 30;
                if (value > m_maxYAxis)
                    value = m_maxYAxis;
                else if (value < m_minYAxis + unit)
                    value = m_minYAxis + unit;
                this.GraphPane.YAxis.Scale.Max = value;
            }
            else if (value > m_minYAxis)
            {
                value = this.GraphPane.YAxis.Scale.Min;
                value += e.Delta * unit / 120;
                if (value < m_minYAxis + unit)
                    value = m_minYAxis + unit;
                this.GraphPane.YAxis.Scale.Min = value;
            }

            this.GraphPane.AxisChange();
            this.Refresh();
        }

        public override void AxisChange()
        {
            GraphPane pane = this.GraphPane;
            XAxis x = pane.XAxis;
            YAxis y = pane.YAxis;
            Harmonic fundamental = new Harmonic();
            FieldInfo fld = fundamental.GetType().GetField(m_graphType.ToString());
            bool isFFT = m_graphType == enumGraphTypes.FourierComponent || m_graphType == enumGraphTypes.NormalizedComponent;

            y.Title.Text = isFFT ? "" : "Degrees";
            y.Type = isFFT ? AxisType.Log : AxisType.Linear;
            if (m_dataSet.Count > 0)
            {
                double min = double.MaxValue;
                double max = 0;
                foreach (HarmonicCollection data in m_dataSet)
                {
                    foreach (Harmonic h in data)
                    {
                        double value = (double)fld.GetValue(h);
                        if (value < min)
                            min = value;
                        if (value > max)
                            max = value;
                    }
                }
                if (max > min)
                {
                    y.Scale.Min = min;
                    y.Scale.Max = max;
                }
                else
                {
                    y.Scale.Min = 1;
                    y.Scale.Max = 10;
                }
            }
            else
            {
                y.Scale.Min = 1;
                y.Scale.Max = 10;
            }

            // Adjust x scale
            if (m_dataSet.Count > 0)
            {
                double min = double.MaxValue;
                double max = 0;
                foreach (HarmonicCollection data in m_dataSet)
                {
                    foreach (Harmonic h in data)
                    {
                        if (h.Frequency < min)
                            min = h.Frequency;
                        if (h.Frequency > max)
                            max = h.Frequency;
                    }
                }
                if (max > min)
                {
                    double range = (max - min);
                    min -= range / 10;
                    max += range / 10;
                    x.Scale.Min = min;
                    x.Scale.Max = max;
                }
                else
                {
                    x.Scale.Min = 0;
                    x.Scale.Max = 20000;
                }
            }
            else
            {
                x.Scale.Min = 0;
                x.Scale.Max = 20000;
            }

            m_minYAxis = y.Scale.Min;
            m_maxYAxis = y.Scale.Max;

            base.AxisChange();
            this.Refresh();
        }
        #endregion

        #region Functions
        public void Clear()
        {
            this.GraphPane.CurveList.Clear();
            m_dataSet.Clear();
            m_currentColor = 0;
            this.Refresh();
        }

        public void AddData(HarmonicCollection data)
        {
            m_dataSet.Add(data);
            this.FillData(data);
            this.AxisChange();
        }

        public void ReloadData(HarmonicCollection data)
        {
            GraphPane pane = this.GraphPane;
            foreach (CurveItem item in pane.CurveList)
            {
                if (item.Tag == data)
                {
                    BarItem bar = (BarItem)item;
                    FillBar(bar);
                    break;
                }
            }
        }

        private void FillData(HarmonicCollection data)
        {
            string title = data.FileName;
            int index = title.LastIndexOf("\\");
            if (index >= 0)
                title = title.Substring(index + 1);
            title += " --- Total THD: " + data.TotalHarmonicDistorsion.ToString("P2");

            BarItem bar = new BarItem(title);
            bar.Tag = data;
            bar.Color = m_graphColors[m_currentColor++];
            if (m_currentColor >= m_graphColors.Length)
                m_currentColor = 0;

            FillBar(bar);
            this.GraphPane.CurveList.Add(bar);
        }

        private void FillBar(BarItem bar)
        {
            HarmonicCollection data = (HarmonicCollection)bar.Tag;
            Harmonic fundamental = data[0];
            FieldInfo fld = fundamental.GetType().GetField(m_graphType.ToString());
            double value0 = (double)fld.GetValue(fundamental);
            bool isFFT = m_graphType == enumGraphTypes.FourierComponent || m_graphType == enumGraphTypes.NormalizedComponent;
            int harmonic = 0;

            bar.Clear();
            
            foreach (Harmonic h in data)
            {
                PointPair point = new PointPair();
                point.X = h.Frequency;
                double value = (double)fld.GetValue(h);
                point.Y = value;
                if (isFFT)
                {
                    double percent = value / value0;
                    string pointToolTip = null;
                    if (isFFT)
                    {
                        if (++harmonic == 1)
                            pointToolTip = "Fundamental\r\n" + percent.ToString("P2");
                        else
                            pointToolTip = "Harmonic " + harmonic + "\r\n" + percent.ToString("P2");
                    }
                    point.Tag = pointToolTip;
                }
                bar.AddPoint(point);
            }
        }
        #endregion
    }
    #endregion
}
